package com.guwan.backend.security;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.guwan.backend.common.Result;
import com.guwan.backend.constant.SecurityConstants;
import com.guwan.backend.util.JwtUtil;
import jakarta.servlet.FilterChain;
import jakarta.servlet.ServletException;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;

/**
 * JWT认证过滤器
 * 负责处理JWT token的验证和用户认证
 */

/**
 * SecurityConfig 和 JwtAuthenticationFilter 之间的关系是，
 * 在SecurityConfig中配置JwtAuthenticationFilter，
 * 使其成为Spring Security过滤器链的一部分。
 * 这样，当请求到达时，JwtAuthenticationFilter会先于Spring Security的其他过滤器执行，以验证JWT的有效性。
 */

@Slf4j
@Component
@RequiredArgsConstructor
public class JwtAuthenticationFilter extends OncePerRequestFilter {

    private final JwtUtil jwtUtil;
    private final UserDetailsServiceImpl userDetailsService;
    private final ObjectMapper objectMapper;

    /**
     * 过滤器主要逻辑
     * 1. 检查是否是白名单路径
     * 2. 验证JWT token
     * 3. 设置认证信息
     */
    @Override
    protected void doFilterInternal(HttpServletRequest request, 
                                  HttpServletResponse response, 
                                  FilterChain chain) throws ServletException, IOException {
        String path = request.getServletPath();
        
        // 如果是白名单路径，直接放行
        if (isPermitPath(path)) {
            chain.doFilter(request, response);
            return;
        }

        try {
            // 验证token
            String authHeader = request.getHeader("Authorization");
            if (authHeader == null || !authHeader.startsWith("Bearer ")) {
                handleAuthenticationError(response, "未登录");
                return;
            }

            String jwt = authHeader.substring(7);
            if (!jwtUtil.validateToken(jwt)) {
                handleAuthenticationError(response, "token已过期或无效");
                return;
            }

            // 设置认证信息
            setAuthentication(request, jwt);

            chain.doFilter(request, response);
        } catch (Exception e) {
            log.error("认证过程发生错误", e);
            handleAuthenticationError(response, "认证失败: " + e.getMessage());
        }
    }

    /**
     * 检查请求路径是否在白名单中
     */
    private boolean isPermitPath(String path) {
        return SecurityConstants.WHITE_LIST.stream()
                .anyMatch(pattern -> matchPath(path, pattern)) ||
               SecurityConstants.STATIC_RESOURCES.stream()
                .anyMatch(pattern -> matchPath(path, pattern));
    }

    /**
     * 路径匹配逻辑
     * 支持通配符 /** 匹配
     */
    private boolean matchPath(String path, String pattern) {
        return pattern.endsWith("/**") 
            ? path.startsWith(pattern.substring(0, pattern.length() - 3))
            : path.equals(pattern);
    }

    /**
     * 设置认证信息到 SecurityContext
     */
    private void setAuthentication(HttpServletRequest request, String jwt) {
        Long userId = jwtUtil.getUserIdFromToken(jwt);
        if (SecurityContextHolder.getContext().getAuthentication() == null) {
            UserDetails userDetails = userDetailsService.loadUserById(userId);
            
            UsernamePasswordAuthenticationToken authentication = 
                new UsernamePasswordAuthenticationToken(
                    userDetails, null, userDetails.getAuthorities());
            authentication.setDetails(
                new WebAuthenticationDetailsSource().buildDetails(request));

            SecurityContextHolder.getContext().setAuthentication(authentication);
        }
    }

    /**
     * 处理认证错误
     * 返回统一的错误响应格式
     */
    private void handleAuthenticationError(HttpServletResponse response, String message) 
            throws IOException {
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.setCharacterEncoding("UTF-8");
        response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
        
        Result<?> result = Result.error(message);
        response.getWriter().write(objectMapper.writeValueAsString(result));
    }
} 