package com.guwan.backend.service.impl;

import com.guwan.backend.service.SmsService;
import com.guwan.backend.service.VerificationService;
import com.guwan.backend.util.RedisUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.stereotype.Service;

import java.util.Random;
import java.util.concurrent.TimeUnit;

@Slf4j
@Service
@RequiredArgsConstructor
public class VerificationServiceImpl implements VerificationService {

    private final RedisUtil redisUtil;
    private final JavaMailSender mailSender;
    private final SmsService smsService;

    private static final String EMAIL_CODE_KEY = "verification:email:";
    private static final String SMS_CODE_KEY = "verification:sms:";
    private static final int CODE_EXPIRE_MINUTES = 5;
    private static final int CODE_LENGTH = 6;
    private static final String SMS_SEND_TIME_KEY = "sms:send:time:";
    private static final String SMS_COUNT_KEY = "sms:count:";
    private static final int MAX_DAILY_SMS_COUNT = 10;

    @Override
    public void sendEmailCode(String email) {
        String code = generateCode();
        // 存储验证码
        redisUtil.set(EMAIL_CODE_KEY + email, code, CODE_EXPIRE_MINUTES, TimeUnit.MINUTES);
        
        // 发送邮件
        SimpleMailMessage message = new SimpleMailMessage();
        message.setTo(email);
        message.setSubject("注册验证码");
        message.setText("您的验证码是：" + code + "，" + CODE_EXPIRE_MINUTES + "分钟内有效。");
        mailSender.send(message);
        
        log.info("发送邮箱验证码：{} -> {}", email, code);
    }

    @Override
    public void sendSmsCode(String phone) {
        // 检查发送频率限制
        checkSendFrequency(phone);
        
        String code = generateCode();
        // 存储验证码
        redisUtil.set(SMS_CODE_KEY + phone, code, CODE_EXPIRE_MINUTES, TimeUnit.MINUTES);
        
        // 发送短信
        smsService.sendVerificationCode(phone, code);
        
        // 记录发送时间
        redisUtil.set(SMS_SEND_TIME_KEY + phone, System.currentTimeMillis(), 24, TimeUnit.HOURS);
        
        log.info("发送短信验证码：{} -> {}", phone, code);
    }

    @Override
    public boolean verifyEmailCode(String email, String code) {
        String key = EMAIL_CODE_KEY + email;
        String savedCode = (String) redisUtil.get(key);
        if (savedCode != null && savedCode.equals(code)) {
            redisUtil.delete(key);
            return true;
        }
        return false;
    }

    @Override
    public boolean verifySmsCode(String phone, String code) {
        String key = SMS_CODE_KEY + phone;
        String savedCode = (String) redisUtil.get(key);
        if (savedCode != null && savedCode.equals(code)) {
            redisUtil.delete(key);
            return true;
        }
        return false;
    }

    private String generateCode() {
        Random random = new Random();
        StringBuilder code = new StringBuilder();
        for (int i = 0; i < CODE_LENGTH; i++) {
            code.append(random.nextInt(10));
        }
        return code.toString();
    }

    private void checkSendFrequency(String phone) {
        // 检查是否在一分钟内重复发送
        String sendTimeKey = SMS_SEND_TIME_KEY + phone;
        Object lastSendTime = redisUtil.get(sendTimeKey);
        if (lastSendTime != null) {
            long timeDiff = System.currentTimeMillis() - (Long) lastSendTime;
            if (timeDiff < 60000) { // 60秒内不能重复发送
                throw new IllegalArgumentException("发送太频繁，请稍后再试");
            }
        }
        
        // 检查当天发送次数
        String countKey = SMS_COUNT_KEY + phone;
        Integer count = (Integer) redisUtil.get(countKey);
        if (count != null && count >= MAX_DAILY_SMS_COUNT) {
            throw new IllegalArgumentException("今日发送次数已达上限");
        }
        
        // 增加发送次数
        if (count == null) {
            redisUtil.set(countKey, 1, 24, TimeUnit.HOURS);
        } else {
            redisUtil.increment(countKey, 1);
        }
    }
} 